//----------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                       (c) 1997 European Space Agency
//
//---------------------------------------------------------------------------
//
//	ImageDisplayer.h:	--- abstract class for image displayer
//
//	DESCRIPTION:
//		A class `ImageDisplayer' is defined which provides a set of functions
//		to control the display of a 2-D image via an external display
//		program.
//
//	CHANGE HISTORY:
//		UL, 1998-10-20: 1.0.0: - first version
//		UL, 2002-09-18: 1.1.0: - various additions to the ImageDisplayer
//								 capabilities
//		UL, 2002-11-13: 1.2.0: - various additions (e.g. deleteRegions)
//
//	AUTHOR:
//		U. Lammers (UL), Tue Oct 20 15:23:28 MET DST 1998
//
//	$Header: /usr/local/cvs/sasdev/task_devel/packages/imgdisplay/src/ImageDisplayer.h,v 1.8 2002/11/13 16:50:22 ulammers Exp $
//

#ifndef IMAGE_DISPLAYER_H
#define IMAGE_DISPLAYER_H

#include <string>
#include <vector>
#include "Tracer.h"
#include "Region.h"

class ImageDisplayer {
public:
	ImageDisplayer() { }
		//
		//	PURPOSE:
		//		Create instance of image displayer
		//

	virtual bool isUp() const = 0;
	virtual bool isDown() const { return !isUp(); }
		//
		//	RETURNS:
		//		T/F: Image displayer is up/not up
		//

	virtual void flipX() const = 0;
	virtual void flipY() const = 0;
		//
		//	PURPOSE:
		//		Flip displayed image around X/Y axis
		//

	typedef enum { X1, X2, X4 } ZoomFactor;
	virtual void zoomIn(ZoomFactor zoomFactor = X1) const = 0;
	virtual void zoomOut(ZoomFactor zoomFactor = X1) const = 0;
		//
		//	PURPOSE:
		//		Zoom in/out by the specified factor
		//

	virtual void displayImage(const std::string &dataSetName) const = 0;
		//
		//	PURPOSE:
		//		Load a new image into the image display frame thereby
		//		destroying the currently displayed one (if any)
		//

	enum CoordType { IMAGE, LINEAR, FK5 };
	class Region : public GeoRegion {
	public:
		Region() {}
		Region(const GeoRegion &r, CoordType coordType = IMAGE)
			: GeoRegion(r), _coordType(coordType) {}

		CoordType coordType() const { return _coordType; }

	private:
		CoordType _coordType;
	};

	virtual void displayRegion(const std::string &regionTable,
							   CoordType coordType = IMAGE) const = 0;
	virtual void displayRegion(const Region &region) const = 0;
		//
		//	PURPOSE:
		//		Visualize the contents of an ASC-region file/Region object by
		//		overlaying the currently displayed image with
		//		region markers
		//
		//	ARGUMENTS:
		//		regionTable: name of ASC-region table to visualize
		//		region	   : region object to display
		//


	virtual std::vector<std::string> regionSelectionAsString(
												CoordType coordType = IMAGE)
	const = 0;
	virtual ::GeoRegion regionSelection(CoordType coordType = IMAGE,
								const std::string &ignoreMarkerColor = "")
	const = 0;
		//
		//	PURPOSE:
		//		Return active region selections in the image display frame
		//		in a string representation/as a GeoRegion object
		//


	virtual void deleteRegions() const = 0;
		//
		//	PURPOSE:
		//		Delete all currently active regions (if any) from the
		//		display
		//


	virtual void down() const = 0;
		//
		//	PURPOSE:
		//		Bring the image display program down.
		//


	virtual std::vector<std::string> receive(const std::string &command)
		const = 0;
	virtual void send(const std::string &command) const = 0;
		//
		//	PURPOSE:
		//		Send an arbitray command to the image displayer and
		//		collect the response; return in a vector of strings
		//


	virtual ~ImageDisplayer() {}
		//
		//	PURPOSE:
		//		Destroy ImageDisplayer object
}; // ImageDisplayer

#endif



