//----------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                       (c) 1997 European Space Agency
//
//---------------------------------------------------------------------------
//
//	SAOtng.cc:	--- implementation of SAOtng class
//
//	CHANGE HISTORY:
//		UL, 1998-10-20: 1.0.0: - first version
//		UL, 1999-02-19: 1.1.0: - workaround for bug in xmgr which prevents
//								 other pipes from being created if a
//								 pipe to a runnig xmgr is open (see below for
//								 more information)
//		UL, 1999-02-19: 1.2.0: - skip comments in region strings from SAOtng
//		UL, 1999-05-06: 1.3.0: - changed all fatal() into error()
//		UL, 1999-09-20: 1.3.1: - do no longer check return value of
//								 'system' used to bring up SAOtng (caused
//								 problem at Leceister)
//		UL, 1999-10-22: 1.3.2: - bug fix in SAOtng::receive(): stderr output
//								 of xpaget was not looked at
//		UL, 1999-10-27: 1.3.3: - in isRunning(): look for either
//								 "XPA$ERROR" (XPA>=2.0) or "ERROR:" (XPA<2.0)
//								 from xpaget to decide if SAOtng is running or
//								 not
//		UL, 2000-01-05: 1.4.0: - only change settings (new colormap/log scale)
//								 when bringing up new ds9
//								 (fixes SPR 1034 : should not reset ds9
//								 settings)
//
//	AUTHOR:
//		U. Lammers (UL), Tue Oct 20 15:59:17 MET DST 1998
//
//

#include <iostream>
#include <streambuf.h>	//	ios_base::sync_with_stdio
#include <cstdlib>
#include <cstdio>
#include <csignal>
#include <unistd.h>
#include "SAOtng.h"
#include "errstr.h"
#include "stringutil.h"

const std::string SAOtng::Xpaset("xpaset");
const std::string SAOtng::Xpaget("xpaget");
const std::string SAOtng::TITENV("LOGNAME");


//
//	switchSignalHandlers: toggle the signal handlers for the signal SIGCLD
//						  ACEgrOpen erroneously registers a handler for
//						  the signal SIGCLD which simply exits the application.
//						  This prevents other pipes from being created and
//						  closed because pclose() emits SIGCLD to signal
//						  that the process who is reading from the pipe
//						  has passed away.
//						  This workaround can go when we switch to
//						  grace-5.0.2 in which libgrace is much cleaner
//						  implemented.
//
static void switchSignalHandlers()
{
	static void (*sigChildHandler)(int) = 0;

	if (sigChildHandler) {
		//	there is a valid signal handler - register it
		std::signal(SIGCLD, sigChildHandler);
		sigChildHandler = 0;
	} else
		//	there is no signal handler
		sigChildHandler = std::signal(SIGCLD, SIG_IGN);
} // switchSignalHandlers()


//	construct SAOtng wrapper object
SAOtng::SAOtng(const std::string &name, bool stopViewer) : ImageDisplayer(),
														   _Saotng(name)
{
	ios::sync_with_stdio();	//	we use popen/pclose
	if (isThere()) {
		_title = "SAOtng";
		char *p = std::getenv(TITENV.c_str());
		if (p)
			_title += "-" + std::string(p);

		if (!isRunning()) {
			if (stopViewer)
				return;

			//	is's not running - try to bring it up
			execBackground(_Saotng + " -title " + _title);
			//	wait till it comes up and is receptive
			unsigned nChecks = 0;
			while (!isRunning() && nChecks++ <= 20)
				sleep(1);
			if (nChecks <= 20) {
				send("zoom 2");
				send("scale log");
				send("colormap Heat");
				send("coords pixels");

				return;
			}
		} else {
			//	is already running
			if (stopViewer)
				down();

			return;
		}
	}

	errstr<<"Could not launch SAOtng. It is either\nnot "
			"installed or inaccessible via the current PATH."
		  <<Sas::error("UnableToLaunchProgram");
} // SAOtng::SAOtng



void SAOtng::flipX() const
{
	send("flip x");
} // SAOtng::flipX()


void SAOtng::flipY() const
{
	send("flip y");
} // SAOtng::flipY()


void SAOtng::zoomIn(ZoomFactor) const
{
	errstr<<"Zooming in not yet implemented - nothing done"
		  <<Sas::warning("NotYetImplemented");
} // SAOtng::zoomIn()


void SAOtng::zoomOut(ZoomFactor) const
{
	errstr<<"Zooming out not yet implemented - nothing done"
		  <<Sas::warning("NotYetImplemented");
} // SAOtng::zoomOut()


void SAOtng::displayImage(const std::string &dataSetName) const
{
	send("file " + dataSetName);
} // SAOtng::displayImage()


void SAOtng::displayRegion(const std::string &, CoordType) const
{
	errstr<<"Displaying regions is not supported with SAOtng - switch "
			"to ds9"<<Sas::warning("UnsupportedOperation");
} // SAOtng::displayRegion()

void SAOtng::displayRegion(const Region &) const
{
	displayRegion("");
} // SAOtng::displayRegion()


std::vector<std::string> SAOtng::regionSelectionAsString(CoordType) const
{
	return receive("regions");
} // SAOtng::regionSelection()


::GeoRegion SAOtng::regionSelection(CoordType, const std::string &) const
{
	errstr<<"Conversion of region to GeoRegion type not supported with "
			"SAOtng - switch to ds9"<<Sas::warning("UnsupportedOperation");

	return GeoRegion();	
} // SAOtng::regionSelection()

void SAOtng::deleteRegions() const
{
	errstr<<"Deletion of regions not supported with SAOtng - switch to ds9"
		  <<Sas::warning("UnsupportedOperation");
} // SAOtng::deleteRegions()


void SAOtng::down() const
{
	send("quit");
} // SAOtng::down()


SAOtng::~SAOtng()
{
} // SAOtng::~SAOtng()


void SAOtng::send(const std::string &command) const
{
	std::string commandLine = Xpaset + " " + _title;
	FILE *xpasetPipe = popen(commandLine.c_str(), "w");
	if (!xpasetPipe ||
		fprintf(xpasetPipe, "%s\n", command.c_str()) <= 0)
		errstr<<"Could not open or write on pipe to `"<<Xpaset
			  <<"' - please note that `"<<Xpaset<<"' must be accessible via"
				" the PATH. Could this be the problem?"
			  <<Sas::warning("SAOtng::send");

	//	see above why whis is necessay
	switchSignalHandlers();
	pclose(xpasetPipe);
	switchSignalHandlers();
} // SAOtng::send()


std::vector<std::string> SAOtng::receive(const std::string &what) const
{
	std::vector<std::string> response;
	std::string commandLine = Xpaget + " " + _title + " " + what + " 2>&1";
	FILE *xpagetPipe = popen(commandLine.c_str(), "r");
	if (xpagetPipe) {
		char s[2048];
		while (std::fgets(s, sizeof(s)-1, xpagetPipe)) {
			if (std::strlen(s) && s[std::strlen(s)-1] == '\n')
				s[std::strlen(s)-1] = '\0';
			if (s[0] != '#')	//	comments
				response.push_back(s);
		}
	} else
		errstr<<"Could not open or write on pipe to `"<<Xpaget
			  <<"' - please note that `"<<Xpaget<<"' must be accessible via"
				" the PATH. Could this be the problem?"
			  <<Sas::warning("SAOtng::receive");

	//	see above why whis is necessay
	switchSignalHandlers();
	pclose(xpagetPipe);
	switchSignalHandlers();

	return response;
} // SAOtng::receive()


bool SAOtng::isRunning() const
{
	std::vector<std::string> v(receive(""));

	if (v.size() == 0)
		return true;

	v = breakup(v[0], " ");

	return v[0] != "XPA$ERROR" && v[0] != "ERROR:";
} // SAOtng::isRunning()


bool SAOtng::isThere() const
{
	return true;
} // SAOtng::isThere()


void SAOtng::execBackground(const std::string &command) const
{
	std::string comm = command + " >/dev/null 2>&1 &";
	std::system(comm.c_str());
} // SAOtng::execBackground()
